// Copy this in the Script/JScript subfolder of your PSPad installation.
// Access it from PSPad in Script/LKR files scripts.
// ---------------------------------------------------------------------------
// LRK utils for PSPad
//
// Version (2014-12-10)
//
// Author, contact ("complaints", questions and comments):
//    Eduard Nickel
//    edo_nick@azet.sk
//    Slovakia
//
// Checks timing in LRK file. Each first word at line has to be delayed 350ms
// to line time [] or equal to line time [] rounded down to centiseconds.
// Checks time order in line and order of line times
// Logs all error into log
// Checks possible timing problems after conversion to LRC format.
// Shifts timing in LRK file. Each time [] and <> shifts with given time in 
// miliseconds (positive or negative). Ommits 0 time.
//

// ---------------------------------------------------------------------------
// JSLint configuration: do not remove.
// This is used to check for potential errors
// in JavaScript sources.
/*jslint adsafe:false */
/*jslint bitwise:true */
/*jslint browser:false */
/*jslint debug:false */
/*jslint eqeqeq:true */
/*jslint evil:false */
/*jslint forin:true */
/*jslint nomen:false */
/*jslint plusplus:true */
/*jslint undef:true */

/*extern
   newEditor
   echo
   inputText
   addMenuItem
*/

// ---------------------------------------------------------------------------
// Integration into PSPad

var module_name = "CheckTimingLRK";
var module_ver = "1.20";

// ---------------------------------------------------------------------------

var editor;
// The current editor.
// Must be initialized by entry points.

// ---------------------------------------------------------------------------
// A procedure entry point procedures must invoke before doing theire work.

function initializeCommand () // Returns nothing
{
   editor = newEditor ();
   editor.assignActiveEditor ();
}

function checkTimeSequenceInLine (line, i) // Returns nothing
{
   var from;
   var firstTime = 0;
   var nextTime;

    from = line.indexOf ("<", from);
    while (from > -1) {
        nextTime = getNumberFromBracket(line, "<>", from);
        from ++;
        from = line.indexOf ("<", from);
        if (nextTime != null) {
            if (!(nextTime > firstTime)) {
                logAddLine("Error at line [" + (i+1) + "] Wrong time order at start time [" + firstTime + "] next time in line is [" + nextTime + "]!");
            }
            firstTime = nextTime;
        }
        //echo ("Debug Line [" + i + "] at [" + from + "]");
    }
}

function getNumberFromBracket (line, brackets, from) // Returns number from brackets
{
   var timeStart; // start position of time bracket, first position should be 1
   var timeEnd; // end position of time bracket
   var num; // number of time

    timeStart = line.indexOf (brackets.substring(0,1), from) + 1;
    //echo ("Debug timeStart[" + timeStart + "]");
    timeEnd = line.indexOf (brackets.substring(1,2), from);       
    //echo ("Debug timeEnd[" + timeEnd + "]");
    if (timeStart != 0 && timeEnd != -1) {
        //echo ("Debug time[" + timeStart + "-" + timeEnd + "] [" + line.substring (timeStart, timeEnd) + "]");
        num = parseInt(line.substring (timeStart, timeEnd));
        //echo ("Debug time[" + timeStart + "-" + timeEnd + "] [" + num + "]");
        if (isNaN(num)) {
            num = null;
        }
    } else {
        num = null;
    }

   return num;
}

function shiftNumberInBracket (line, brackets, shift, from) // Returns nothing
{
   var timeStart; // start position of time bracket, first position should be 1
   var timeEnd; // end position of time bracket
   var num; // number of time
   var newLine; // changed line output

    timeStart = line.indexOf (brackets.substring(0,1), from) + 1;
    //echo ("Debug timeStart[" + timeStart + "]");
    timeEnd = line.indexOf (brackets.substring(1,2), from);       
    //echo ("Debug timeEnd[" + timeEnd + "]");
    if (timeStart != 0 && timeEnd != -1) {
        //echo ("Debug time[" + timeStart + "-" + timeEnd + "] [" + line.substring (timeStart, timeEnd) + "]");
        num = parseInt(line.substring (timeStart, timeEnd));
        // avoid to substract from zero
        if (num > 0) { // move only not zero times || parseInt(shift) > 0
            num = (num + parseInt(shift));
        }
        //echo ("Debug time[" + timeStart + "-" + timeEnd + "] [" + num + "]");
        if (isNaN(num)) {
            newLine = line;
        }
        else {
            newLine = line.substring(0,timeStart) + num.toString() + line.substring(timeEnd, line.length);
        }
    } else {
        newLine = line;
    }

   return newLine;
}

// ---------------------------------------------------------------------------

function checkTimingLRK () // Returns nothing
{
   // ------------------------------------------------------------------------
   // Main

   var filename = getVarValue("%FileName%");
   var text; // String
   var lines;
   var line; // item from lines at i
   var lnTime; // string of line time
   var firtLnTime = 0; // first string of line time
   var firstWordTime; // string of first word time
   var firstWordTimeRound; // string of first word time
   var delay = 350; // delay time in miliseconds
   var firstVerseSwitch = true; // 
   var changeFlag = false; // indicate correction was made

   // check right file type
   if (filename.substring(filename.length-3,filename.lengthlength).toUpperCase() != "LRK") {
       echo ("File " + filename + " is not LRK file!");
       return;
   }
   
   logClear();
   initializeCommand ();

   // Get selection content
   //ensureSelectionCoverLinesInWhole ();
   //text = editor.selText ();
   text = editor.text ();
   lines = text.split ("\n");
   // Do the useful stuff
   for (i = 0; i < lines.length; i += 1) {
      line = lines[i];

      lnTime = getNumberFromBracket(line, "[]", 0);
      firstWordTime = getNumberFromBracket(line, "<>", 0);
      firstWordTimeRound = Math.floor(firstWordTime/10)*10;
      
      if (line.indexOf ("[") > 0) {
          lines[i] = line.substring (line.indexOf ("["), line.length);
          changeFlag = true;
      } 
      
      //echo ("Debug Line [" + (i+1) + "] LnTime [" + lnTime + "] FirstWordTime [" + firstWordTime + "]");
      if (lnTime != null && firstWordTime != null) {
          //echo ("Debug Line [" + (i+1) + "] LnTime [" + (lnTime+delay) + "] FirstWordTime [" + firstWordTimeRound + "]");
          if (firstWordTimeRound != (lnTime+delay)) {
              if (firstVerseSwitch == false || firstWordTimeRound != lnTime) {
                  logAddLine("Error at line [" + (i+1) + "] Line Time [" + lnTime + "] Time of First Word [" + firstWordTimeRound + "]");
              }
          }
          firstVerseSwitch = false;
      }
      else if (lnTime != null && firstWordTime == null) {
          firstVerseSwitch = true;
          var time = lnTime.toString();
          var milis = parseInt(time.substring(time.length-3, time.length));
          if (Math.round(milis/10) == 100) {
              logAddLine("Error at line [" + (i+1) + "] Line Time [" + lnTime + "] possibly lead to wrong timing after conversion to LRC!");
          }
      }
      else if (lnTime == null && firstWordTime != null) {
          logAddLine("Error at line [" + (i+1) + "] Line Time is not set!");
      }
      
      // check for time sequence in line
      if (firstWordTime != null) {
          checkTimeSequenceInLine(line, i);
      }

      //echo ("Debug Line [" + (i+1) + "] LnTime [" + (lnTime) + "] firtLnTime [" + firtLnTime + "]");
      if (lnTime != null) {
          if (lnTime <= firtLnTime) {
              if (!(lnTime == 0 && firtLnTime == 0)) {
              logAddLine("Error at line [" + (i+1) + "] Wrong time order [" + lnTime + "] previous Line time is [" + firtLnTime + "]!");
              }
          }
          firtLnTime = lnTime;
      }
   }

   if (changeFlag) {
       text = lines.join ("\n");
       editor.text (text);
   }
}

function shiftTimingLRK () // Returns nothing
{
   var filename = getVarValue("%FileName%");
   var text; // String
   var lines;
   //var line; // item from lines at i
   var lnTime; // string of line time
   var shift;
   var cancelValue = "[--canceled--]";
   var from;

   // check right file type
   if (filename.substring(filename.length-3,filename.lengthlength).toUpperCase() != "LRK") {
       echo ("File " + filename + " is not LRK file!");
       return;
   }
   initializeCommand ();

   // Get selection content
   text = editor.text ();
   lines = text.split ("\n");

   // Get the time shift
   shift = inputText ("Input shift in miliseconds", "", cancelValue);
   if (shift === cancelValue) {
      return;
   }

   // Do the useful stuff
   for (i = 0; i < lines.length; i += 1) {
      //line = lines[i];
      lines[i] = shiftNumberInBracket(lines[i], "[]", shift, 0);

      from = lines[i].indexOf ("<", from);
      while (from > -1) {
          lines[i] = shiftNumberInBracket(lines[i], "<>", shift, from);
          from ++;
          from = lines[i].indexOf ("<", from);
          //echo ("Debug Line [" + i + "] at [" + from + "]");
      }
      
   }

   text = lines.join ("\n");
   editor.text (text);
}

// ---------------------------------------------------------------------------
// Integration into PSPad

function Init ()
{
   addMenuItem ("Check timing", "LRK utils", "checkTimingLRK", "Shift+Ctrl+C");
   addMenuItem ("Shift timing", "LRK utils", "shiftTimingLRK", "Shift+Ctrl+F");
}
